"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const graphql_1 = require('graphql');
const jsforce_1 = require('jsforce');
class GraphQLSalesforce {
    static makeSchema(username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            const sObjectDescribePromises = [];
            yield GraphQLSalesforce.connection.login(username, password);
            for (const sObject of (yield GraphQLSalesforce.connection.describeGlobal()).sobjects)
                sObjectDescribePromises.push(GraphQLSalesforce.connection.sobject(sObject.name).describe());
            yield Promise.all(sObjectDescribePromises);
            return new graphql_1.GraphQLSchema({
                query: new graphql_1.GraphQLObjectType({
                    name: 'Salesforce',
                    fields: GraphQLSalesforce.makeObjectTypes()
                })
            });
        });
    }
    static makeObjectTypes() {
        const objectTypes = {};
        for (const sObject of GraphQLSalesforce.connection.describeGlobal$().sobjects)
            objectTypes[sObject.name] = {
                type: new graphql_1.GraphQLObjectType({
                    name: sObject.name,
                    fields: GraphQLSalesforce.makeScalarTypes(sObject.name)
                }),
                description: sObject.label
            };
        return objectTypes;
    }
    static makeScalarTypes(sObject) {
        const scalarTypes = {};
        for (const field of GraphQLSalesforce.connection.sobject(sObject).describe$().fields)
            scalarTypes[field.name] = {
                type: graphql_1.GraphQLString,
                description: field.label
            };
        return scalarTypes;
    }
}
GraphQLSalesforce.connection = new jsforce_1.Connection();
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = GraphQLSalesforce;
//# sourceMappingURL=GraphQLSalesforce.js.map